@@ -227,6 +227,17 @@ def _set_bill_opacities(
             ))
 
 
+@app.command(name='reset-bill-opacities')
+def _reset_bill_opacities(port: Path = PortArg) -> None:
+    """
+    The command is used to reset the reference value in order to detect double notes.
+    """
+    with PuloonExceptionHandler():
+        with Serial(str(port.resolve())) as serial:
+            puloon = PuloonLCDM4000(serial)
+            typer.echo(puloon.reset_bill_opacities())
+
+
 @app.command(name='get-bill-opacities')
 def _get_bill_opacities(port: Path = PortArg) -> None:
     """The command will get the OPACITY data from each cassette."""
@@ -323,6 +334,17 @@ def _set_bill_lengths(
             ))
 
 
+@app.command(name='reset-bill-lengths')
+def _reset_bill_lengths(port: Path = PortArg) -> None:
+    """
+    The command is used to reset the reference value in order to detect double notes.
+    """
+    with PuloonExceptionHandler():
+        with Serial(str(port.resolve())) as serial:
+            puloon = PuloonLCDM4000(serial)
+            typer.echo(puloon.reset_bill_lengths())
+
+
 @app.command(name='get-bill-lengths')
 def _get_bill_lengths(port: Path = PortArg) -> None:
     """The command will get to saved length data for each cassette."""
