@@ -102,6 +102,13 @@ class CommandMessage:
 
 
 class ResponseMessage:
+    """
+    Message protocol is dependent on Command and Response of message and
+    has a little difference up to the function with specific format.
+
+    BCC can be gotten through exclusive or (XOR) from the start of each message to ETX except BCC.
+    """
+
     SOH: Final[int] = 0x01  # Start of Header
     ID: Final[int] = 0x30  # Communications ID
     STX: Final[int] = 0x02  # Start of Text
@@ -319,7 +326,7 @@ class DispenseResponse(ResponseMessage):
             self.EXIT2, self.REJECT2, self.TYPE2,
             self.EXIT3, self.REJECT3, self.TYPE3,
             self.EXIT4, self.REJECT4, self.TYPE4,
-            *[self.RSV] * 9
+            *self._origin[-2 - 9:-2]
         )
 
 
@@ -704,10 +711,14 @@ class PuloonLCDM4000:
         cmd = cmd_class(*args)
 
         for _ in range(3):
+            if __debug__:
+                print('CMD: ' + ' '.join(hex(b) for b in cmd.bytes()))
             self.port.write(cmd.bytes())
             self.port.timeout = self.ack_timeout[1]
 
             ack = self.port.read(1)
+            if __debug__:
+                print('ACK: ' + ' '.join(hex(b) for b in ack))
             if len(ack) > 0 and self.ack == ack[0]:
                 break
         else:
@@ -716,14 +727,20 @@ class PuloonLCDM4000:
         for _ in range(3):
             self.port.timeout = self.response_delay[1]
             data = self.port.read(cmd.response_size())
+            if __debug__:
+                print('RSP: ' + ' '.join(hex(b) for b in data))
             resp = resp_class(*data)
             if resp.is_valid():
                 self.port.write(chr(self.ack).encode('ascii'))
                 eot = self.port.read(1)
+                if __debug__:
+                    print('EOT: ' + ' '.join(hex(b) for b in eot))
                 if len(eot) > 0 and self.eot == eot[0]:
                     if resp.ERROR != NO_ERROR:
                         raise PuloonException(resp.ERROR - NO_ERROR, response=resp)
                     break
+            if __debug__:
+                print(resp)
             self.port.write(chr(self.nak).encode('ascii'))
         else:
             raise PuloonException(RESPONSE_TIMEOUT)
@@ -731,18 +748,43 @@ class PuloonLCDM4000:
         return resp
 
     def reset(self):
+        """
+        The reset will cause the dispenser reset by software. Therefore, there is no response for this command.
+        (Cf.) When RESET is transmitted, it would take 2 seconds for dispenser to initialize all status.
+        Therefore, the next command would be sent after the initialization.
+        """
         cmd = ResetCommand()
         self.port.write(cmd.bytes())
         sleep(2)
 
     def status(self) -> StatusResponse:
+        """This command shows the current sensor status and the configuration of cassette in the top position."""
         return self._run(StatusCommand, StatusResponse)
 
     def purge(self) -> PurgeResponse:
+        """
+        PURGE will cause the dispenser to purge the transport of all bills from four cassettes and
+        to move the bills in the path to the reject tray. This command will not be required for normal operation.
+        However, in case of abnormal termination such as sudden power-off by external cause,
+        the command will be useful to remove the notes.
+        A successful PURGE operation will move any bills in the transport to the reject tray but
+        if the note would be left in the EXIT area, it may be dispensed.
+
+        PURGE will perform the repetitive routine of FORWARD/BACKWARD FEED itself and cause the damage to notes.
+        It will not recover errors completely by JAM or already terminated DISP (dispense) command.
+        Therefore, it is recommended to use carefully.
+        """
         return self._run(PurgeCommand, PurgeResponse)
 
     def dispense(self, qty1: int = 0, qty2: int = 0, qty3: int = 0, qty4: int = 0,
                  to1: bool = False, to2: int = 0) -> DispenseResponse:
+        """
+        The command will cause to dispenser the requested number of notes from the requested cassette.
+        It will check thickness and length of notes, which are individually referred to the specified OPACITY and LENGTH,
+        and then decide whether the notes are dispensed or rejected. During the process,
+        other parameters such as the required distance between notes and
+        the skew of notes will give influence on dispensing and rejecting.
+        """
         assert all(0 <= q <= 40 for q in (qty1, qty2, qty3, qty4)), "Quantity should be in range of 0 and 40"
         assert 0 < sum((qty1, qty2, qty3, qty4)) <= 40, "Quantity should be at least 1 and maximum 40 in total"
         assert 0 <= to2 <= 9, "Timeout should be in range of 0 and 9"
@@ -756,6 +798,10 @@ class PuloonLCDM4000:
 
     def test_dispense(self, qty1: int = 0, qty2: int = 0, qty3: int = 0, qty4: int = 0,
                       to1: bool = False, to2: int = 0) -> TestDispenseResponse:
+        """
+        The command will cause to reject the specified number of notes from the cassette to the reject tray.
+        All the specified notes will move into the reject tray.
+        """
         assert all(0 <= q <= 40 for q in (qty1, qty2, qty3, qty4)), "Quantity should be in range of 0 and 40"
         assert 0 < sum((qty1, qty2, qty3, qty4)) <= 40, "Quantity should be at least 1 and maximum 40 in total"
         assert 0 <= to2 <= 9, "Timeout should be in range of 0 and 9"
@@ -768,9 +814,18 @@ class PuloonLCDM4000:
         )
 
     def last_status(self) -> LastStatusResponse:
+        """
+        The command will request to resend the results to the last operation commands
+        such as PURGE, DISPENSE and TEST DISPENSE.
+        Therefore, it is effective only when the prior operation was performed.
+        """
         return self._run(LastStatusCommand, LastStatusResponse)
 
     def sensor_diagnostics(self, pos: int = 1) -> SensorDiagnosticsResponse:
+        """
+        The command will cause to dispense 5 notes from the designated cassette as if “TEST DISPENSE” will do.
+        The notes are moved to reject tray and the measured OPACITY, LENGTH and SOLENOID TIME of the last note is returned.
+        """
         assert 1 <= pos <= 4, "Dispenser has only 4 cassette"
         return self._run(SensorDiagnosticsCommand, SensorDiagnosticsResponse, pos + 0x30)
 
@@ -778,6 +833,14 @@ class PuloonLCDM4000:
                            opac2_high: int = 15, opac2_low: int = 1,
                            opac3_high: int = 15, opac3_low: int = 1,
                            opac4_high: int = 15, opac4_low: int = 1) -> SetBillOpacitiesResponse:
+        """
+        The command is used to save the reference value in order to detect double notes.
+        When the data is changed, it will be saved in the memory of EEPROM and then efficient for the next transaction.
+        In case of power on/off, the value continues to be used.
+        However, when the electricity trouble causes the saved data damaged (wrong check sum on EEPROM),
+        the criterion is set to initial value again.
+        Therefore, it is recommended for user to check the value of the saved value of OPACITY when it is turned on.
+        """
         assert all(1 <= v <= 15 for v in (
             opac1_high, opac1_low, opac2_high, opac2_low,
             opac3_high, opac3_low, opac4_high, opac4_low
@@ -787,11 +850,30 @@ class PuloonLCDM4000:
                                opac3_high, opac3_low, opac4_high, opac4_low)
         ))
 
+    def reset_bill_opacities(self) -> SetBillOpacitiesResponse:
+        """
+        The command is used to reset the reference value in order to detect double notes.
+        """
+        return self._run(SetBillOpacitiesCommand, SetBillOpacitiesResponse,
+                         0x00, 0x00, 0x00, 0x00,
+                         0x00, 0x00, 0x00, 0x00)
+
     def get_bill_opacities(self) -> GetBillOpacitiesResponse:
+        """The command will get the OPACITY data from each cassette."""
         return self._run(GetBillOpacitiesCommand, GetBillOpacitiesResponse)
 
     def set_bill_dispense_order(self, order1: int = 1, order2: int = 2,
                                 order3: int = 3, order4: int = 4) -> SetBillDispenseOrderResponse:
+        """
+        The command will define the bill dispense order from multi-cassettes.
+        The default order is to pick bills from top cassette first, then second cassette and so on.
+        The invalid assignment of parameter will cause an error and not be saved.
+        When the data is changed, it will be saved in the memory of EEPROM and then efficient for the next transaction.
+        In case of power on/off, the value continues to be used. However,
+        when the electricity trouble causes the saved data damaged (wrong check sum on EEPROM),
+        the criterion is set to initial value again.
+        Therefore, it is recommended for user to check the value of the saved bill dispenser order when it is turned on.
+        """
         assert all(1 <= v <= 4 for v in (order1, order2, order3, order4)), \
             "Order value should be in range of 1 and 4"
         assert len({order1, order2, order3, order4}) == 4, "Order values should be unique"
@@ -799,12 +881,21 @@ class PuloonLCDM4000:
                          *(v + 0x30 for v in (order1, order2, order3, order4)))
 
     def get_bill_dispense_order(self) -> GetBillDispenseOrderResponse:
+        """The command will get the bill dispense order data."""
         return self._run(GetBillDispenseOrderCommand, GetBillDispenseOrderResponse)
 
     def set_bill_lengths(self, leng1_high: int = 15, leng1_low: int = 1,
                          leng2_high: int = 15, leng2_low: int = 1,
                          leng3_high: int = 15, leng3_low: int = 1,
                          leng4_high: int = 15, leng4_low: int = 1) -> SetBillLengthsResponse:
+        """
+        The command is used to save the reference value in order to detect double notes.
+        When the data is changed, it will be saved in the memory of EEPROM and then efficient for the next transaction.
+        In case of power on/off, the value continues to be used. However,
+        when the electricity trouble causes the saved data damaged (wrong check sum on EEPROM),
+        the criterion is set to initial value again.
+        Therefore, it is recommended for user to check the value of the saved value of LENGTH when it is turned on.
+        """
         assert all(1 <= v <= 15 for v in (
             leng1_high, leng1_low, leng2_high, leng2_low,
             leng3_high, leng3_low, leng4_high, leng4_low
@@ -814,5 +905,14 @@ class PuloonLCDM4000:
                                leng3_high, leng3_low, leng4_high, leng4_low)
         ))
 
+    def reset_bill_lengths(self) -> SetBillLengthsResponse:
+        """
+        The command is used to reset the reference value in order to detect double notes.
+        """
+        return self._run(SetBillLengthsCommand, SetBillLengthsResponse,
+                         0x00, 0x00, 0x00, 0x00,
+                         0x00, 0x00, 0x00, 0x00)
+
     def get_bill_lengths(self) -> GetBillLengthsResponse:
+        """The command will get to saved length data for each cassette."""
         return self._run(GetBillLengthsCommand, GetBillLengthsResponse)
